(function() {
	/**
	 * This object houses commonly used/shared function for the builder part of the application.
	 *
	 * @param $http
	 * @param contextualData
	 * @param $log
	 * @returns {{checkIfNameExists: checkFormAvailability}}
	 * @constructor
	 */
	var CommonUseFactory = function($http, contextualData, $log) {
		return {
			checkIfNameExists: checkFormAvailability,
			logicResolver: logicResolver,
			valueResolver: valueResolver,
			placeholderResolver: placeholderResolver,
			inputsizeResolver: inputsizeResolver,
			nameResolver: nameResolver,
			labelResolver: labelResolver,
			parseDate: parseDate,
			createMoment: createMoment
		};
		
		function checkFormAvailability(newName, flag, formIdentifier) {
			var self = this; //references the object from the context of which the call is made!
			if (!_.isUndefined(newName) && newName != '') {
				var data = {
					formName: newName,
					siteIdentifier: contextualData.siteIdentifier
				};

				if (!_.isUndefined(formIdentifier)) {
					data.formIdentifier = formIdentifier;
				}
				$http({
					url: contextualData.context + '/modules/formfactory/builder/checkformnameavailability/' + contextualData.locale,
					method: 'POST',
					data: data
				}).then(function(response) {
					self[flag] = response.data.available;
				});
			}
			else {
				self[flag] = false;
			}
		}

		function valueResolver(args) {
			console.log("Resolving: ");
			//console.log(args);
		}

		function placeholderResolver(input) {
			if ("placeholder" in input) {
				return input.placeholder;
			}
			return null;
		}

		function logicResolver(input, form) {
			if (input !== undefined) {
				if(!('ffVisible' in form[input.name])) {
					return true;
				}
				return form[input.name].ffVisible;
			}
			return true;
		}

		function inputsizeResolver(input) {
			if ("inputsize" in input) {
				return input.inputsize;
			}
			return null;
		}

		function nameResolver(input) {
			if ("name" in input) {
				return input.name;
			}
			return null;
		}

		function labelResolver(input) {
			if ("label" in input) {
				return input.label;
			}
			return null;
		}

		function parseDate(value, format) {
			if (_.isUndefined(value) || value === null || (value instanceof Date && isNaN(value.getTime()))) {
				return false;
			}
			format = format != null ? format : 'YYYY-MM-DD';
			//Check if this is an instance of Date object and that it is not invalid.
			if (value instanceof Date) {
				value = moment(value.toISOString()).format(format);
			} else if (moment.isMoment(value)) {
				return true;
			}

			//Check if this is a ISO_8601 string.
			if (typeof value === 'string') {
				var components = value.split('-');
				if (components.length === 3) {
					components = components[2].split(':');
					if (components.length === 3) {
						components = components[2].split('.');
						if (components.length === 2) {
							return true;
						}
					}
				}
			}

			return validateFormat(value, ['-', '/'], format).isValid;
		}

		function createMoment(value, formatSeparators, format) {
			if (_.isUndefined(value) || value === null) {
				return null;
			}
			if (value instanceof Date) {
				return moment(value.toISOString());
			} else if (moment(value, moment.ISO_8601, true).isValid()) {
				return moment(value, moment.ISO_8601, true).toISOString();
			}
			format = format != null ? format : 'YYYY-MM-DD';
			var formatSeparators = _.isUndefined(formatSeparators) || formatSeparators === null ? ['-', '/'] : (_.isString(formatSeparators) ? [].push(formatSeparators) : formatSeparators);

			var validatedDate = validateFormat(value, formatSeparators, format);
			return validatedDate.isValid ? validatedDate.date.toISOString() : null;
		}

		function validateFormat(value, formatSeparators, format) {
			var validatedDate = {
				isValid: false,
				date: null
			};
			for (var i in formatSeparators) {
				var sections = value.toString().split(formatSeparators[i]);
				var formatSections = format.split(formatSeparators[i]);
				if (sections.length == formatSections.length) {
					validatedDate.isValid = true;
					var formatSectionIndexes = {
						year: null,
						month: null,
						date: null
					};
					for (var j in sections) {
						if (sections[j] === undefined || sections[j] === "" || sections[j] === null || isNaN(sections[j]) || parseInt(sections[j]) < 0 || (sections[j].toString().length !== formatSections[j].length)) {
							validatedDate.isValid = false;
							break;
						}
						if (formatSections[j].toLowerCase().indexOf('y') !== -1) {
							formatSectionIndexes.year = j;
						} else if (formatSections[j].toLowerCase().indexOf('m') !== -1) {
							formatSectionIndexes.month = j;
						} else if (formatSections[j].toLowerCase().indexOf('d') !== -1) {
							formatSectionIndexes.date = j;
						}
						sections[j] = parseInt(sections[j], 10);
						//make sure each is a number.
					}
					if (!validatedDate.isValid) {
						break;
					}

					validatedDate.date = moment();
					if(!_.isNull(formatSectionIndexes.year)) {
						validatedDate.date.year(sections[formatSectionIndexes.year]);
					}
					if(!_.isNull(formatSectionIndexes.month)) {
						validatedDate.date.month(sections[formatSectionIndexes.month] - 1);
					}
					if(!_.isNull(formatSectionIndexes.date)) {
						validatedDate.date.date(sections[formatSectionIndexes.date]);
					}
					//Make sure the date remains the same after the creation.
					var formatString = "";
					for(var sec in formatSections) {
						formatString += formatSections[sec].substring(0,1);
					}
					validatedDate.isValid = validatedDate.date.format(formatString) == sections.join('-');
				}
			}
			return validatedDate;
		}
	};

	CommonUseFactory.$inject = ['$http', 'contextualData', '$log'];

	angular
		.module('formFactory.commonUseFactory', [])
		.factory('ffCommonUseFactory', CommonUseFactory);
})();(function() {
	var dataFactory = function($http, $log) {
		return {
			getData: getData,
			customRequest: customRequest
		};

		function getData(url) {
			return $http.get(url)
				.then(getDataComplete)
				.catch(getDataFailed);
		}

		function customRequest(obj) {
			return $http(obj).then(getDataComplete);
		}

		function getDataComplete(response) {
			return response.data;
		}

		function getDataFailed(error) {
			$log.error('XHR Failed to execute your request.' + error.data);
			return error.data;
		}
	};

	dataFactory.$inject = ['$http', '$log'];

	angular
		.module('formFactory.dataFactory', [])
		.factory('ffDataFactory', dataFactory);
})();(function() {
    var parseDate = function($log, ffCommonUseFactory, $timeout) {

        var directive = {
            restrict: 'A',
            require: ['^ngModel'],
            link: linkFunction
        };

        return directive;

        function linkFunction(scope, el, attr, ctrls) {
            var model               = ctrls[0];
            var hasParserExecuted   = false;
            var parserInProgress    = false;
            var viewValue;
            var AVAILABLE_FORMAT_SEPARATORS = ['-', '\/'];
            var FORMAT_TEMPLATE = attr.format != null ? attr.format : 'YYYY-MM-DD';
            var MAX_CHARACTERS = getMaxCharacters(FORMAT_TEMPLATE);
            var FORMAT_SEPARATOR_PROVIDED   = !_.isUndefined(attr.ffParseDate) && attr.ffParseDate !== null && attr.ffParseDate.trim().length > 0;
            var REGEXP_FORMAT_SEPARATOR     = FORMAT_SEPARATOR_PROVIDED ? attr.ffParseDate : '-|\/';
            var FORMAT_REGEXP_PATTERN       = '(^[0-9]+)(?=[' + REGEXP_FORMAT_SEPARATOR + ']+)[' + REGEXP_FORMAT_SEPARATOR + '](?=[0-9]+)[0-9]+(?=[' + REGEXP_FORMAT_SEPARATOR + ']+)[' + REGEXP_FORMAT_SEPARATOR + '](?=[0-9]+)[0-9]+(?=[0-9]*)' +
                '|(^[0-9]+)(?=[' + REGEXP_FORMAT_SEPARATOR + ']+)[' + REGEXP_FORMAT_SEPARATOR + '](?=[0-9]+)[0-9]+(?=[' + REGEXP_FORMAT_SEPARATOR + ']+)[' + REGEXP_FORMAT_SEPARATOR + ']' +
                '|(^[0-9]+)(?=[' + REGEXP_FORMAT_SEPARATOR + ']+)[' + REGEXP_FORMAT_SEPARATOR + '](?=[0-9]+)[0-9]+' +
                '|(^[0-9]+)(?=[' + REGEXP_FORMAT_SEPARATOR + ']+)[' + REGEXP_FORMAT_SEPARATOR + ']' +
                '|(^[0-9]+)';
            var FORMAT_REGEXP               = RegExp(FORMAT_REGEXP_PATTERN, 'g');
            var REPLACE_REGEXP              = RegExp(REGEXP_FORMAT_SEPARATOR, 'g');

            //This watch is used to get the back view value based on the initialized view value.
            //It is registered once a value is set, or if the parser is called before a value is set.
            var initializeViewValue = scope.$watch(function() {
                return model.$viewValue;
            }, function (value) {
                if (hasParserExecuted) {
                    initializeViewValue();
                    return;
                }
                viewValue = value;
                initializeViewValue();
            });

            /**
             *
             * @param value Date, Moment ISO String, value
             * @param formatSeparator the separator that the format should use
             * @returns String representing the date's format
             */
            function getFormattedRepresentation(value, formatSeparator) {
                if (value instanceof Date) {
                    //Create format using the iso string received from the date object with the specified separator.
                    return moment(value.toISOString(), moment.ISO_8601, true).format(FORMAT_TEMPLATE).replace(/-/g, formatSeparator);
                } else if (typeof value === 'string' && moment(value, moment.ISO_8601, true).isValid()) {
                    //Create format using the iso string with the specified separator.
                    return moment(value, moment.ISO_8601, true).format(FORMAT_TEMPLATE).replace(/-/g, formatSeparator);
                } else {
                    //Just return the given value
                    return value;
                }
            }

            /**
             * This function will return the first separator it can find, If no separator
             * can be found, '-' will be returned.
             * @param value the date which contains the separators to be extracted.
             * @returns String, The first valid seperator that is found. Defaults to '-'.
             */
            function getFormatSeparator (value) {
                if (FORMAT_SEPARATOR_PROVIDED) {
                    return REGEXP_FORMAT_SEPARATOR
                } else {
                    //Retrieve the format.
                    //use the first separator that is found
                    var formatSeparator = '-';
                    if (!_.isUndefined(value) && value !== null && !(value instanceof Date)) {
                        var separatorIndex = -1;
                        for (var i in AVAILABLE_FORMAT_SEPARATORS) {
                            var currentSeparatorIndex = value.indexOf(AVAILABLE_FORMAT_SEPARATORS[i]);
                            if (currentSeparatorIndex !== -1 && (separatorIndex === -1 || currentSeparatorIndex < separatorIndex)) {
                                separatorIndex = currentSeparatorIndex;
                                formatSeparator = AVAILABLE_FORMAT_SEPARATORS[i];
                            }
                        }
                    }
                    return formatSeparator
                }
            }

            /**
             * Sets the view value using a moment ISO_8601 string or Date object and renders it.
             * @param value - The value that will be used to create the view value
             * @param separator - the separator to be used in creating the view value
             * @returns String - ISO_8601 Date representation
             */
            function setViewValueFromDateOrMoment(value, separator) {
                var m = value instanceof Date ? moment(viewValue.toISOString(), moment.ISO_8601, true) : moment(viewValue, moment.ISO_8601, true);
                value = m.format(FORMAT_TEMPLATE);
                if (separator !== '-') {
                    value.replace(/-/g, separator);
                }
                model.$setViewValue(value);
                model.$render();
                return m.toISOString();
            }
            
            function getMaxCharacters(format) {
                var maxCharacters;
                for (var i in AVAILABLE_FORMAT_SEPARATORS) {
                    if (format.indexOf(AVAILABLE_FORMAT_SEPARATORS[i]) !== -1) {
                        var sections = format.split(AVAILABLE_FORMAT_SEPARATORS[i]);
                        //Initialize max characters to size of array -1 for the amount of separators
                        maxCharacters = sections.length - 1;
                        for (var j in sections) {
                            maxCharacters += sections[j].length;
                        }
                        break;
                    }
                }
                return maxCharacters;
            }

            model.$parsers.push(function(value) {
                //This flag is necessary in order to store the initial view value when
                //the model is first initialized (if it is initialized with a value)
                hasParserExecuted = true;
                if (parserInProgress) {
                    return value;
                }
                //This flag is important to not reparse a value that is currently in the process of being parsed.
                parserInProgress = true;
                //Ignore undefined or null
                if (_.isUndefined(value) || value === null) {
                    parserInProgress = false;
                    viewValue = value;
                    return value;
                }
                //check if the value is a date object
                if (value instanceof Date) {
                    model.$setViewValue(getFormattedRepresentation(value, getFormatSeparator(viewValue)));
                    viewValue = value;
                    model.$render();
                    parserInProgress = false;
                    return moment(value.toISOString(), moment.ISO_8601, true).toISOString();
                }
                //Check that the characters are numbers and separators
                //Get value based on regular exp.
                var matchedValue = value.match(FORMAT_REGEXP);
                //Ensure the value is valid.
                if (matchedValue === null) {
                    value = viewValue;
                    model.$setViewValue(getFormattedRepresentation(value, getFormatSeparator(value)));
                    model.$render();
                    parserInProgress = false;
                    return value;
                }

                var formatSeparator = getFormatSeparator(value);

                //If the new value is a match we set it, otherwise we fallback to previously parsed value
                value = viewValue !== null && matchedValue[0].length < viewValue.length && matchedValue[0].length != value.length ? viewValue : matchedValue[0];
                value = value.replace(REPLACE_REGEXP, formatSeparator);
                if (value.toString().length > MAX_CHARACTERS) {
                    if (ffCommonUseFactory.parseDate(value, FORMAT_TEMPLATE)) {
                        //check if this could be an iso representation first before trying to give it a format.
                        var momentFormat = formatSeparator !== '-' ? angular.copy(FORMAT_TEMPLATE).replace(/-/g, formatSeparator) : FORMAT_TEMPLATE;
                        var m = moment(ffCommonUseFactory.createMoment(value, FORMAT_SEPARATOR_PROVIDED ? REGEXP_FORMAT_SEPARATOR : ['-', '/'], FORMAT_TEMPLATE), moment.ISO_8601, true);
                        value = m.toISOString();
                        viewValue = value;
                        model.$setViewValue(m.format(momentFormat));
                        model.$render();
                    } else {
                        if (viewValue instanceof Date || moment(viewValue, moment.ISO_8601, true).isValid()) {
                            value = setViewValueFromDateOrMoment(viewValue, formatSeparator);
                        } else {
                            value = viewValue;
                            model.$setViewValue(value);
                            model.$render();
                            value = moment(value, angular.copy(FORMAT_TEMPLATE).replace(/-/g, formatSeparator), true).toISOString();
                        }
                    }
                } else {
                    viewValue = value;
                    model.$setViewValue(value);
                    model.$render();
                    //Create moment ISO_8601 string only if this is a valid date, otherwise it is possible
                    //that moment will create an ISO_8601 from an incomplete date.
                    if (ffCommonUseFactory.parseDate(value, FORMAT_TEMPLATE)) {
                        value = moment(value, angular.copy(FORMAT_TEMPLATE).replace(/-/g, formatSeparator)).toISOString();
                    }
                }
                //This is necessary in order to remove the validation that is added by ui Datepicker
                //which is not necessary and interferes with our form validity.
                $timeout(function () {
                    model.$setValidity('date', true);
                });
                //Finished parsing so unset the flag.
                parserInProgress = false;
                return value;
            });
        }
    };


    angular
        .module('formFactory')
        .directive('ffParseDate',  ['$log', 'ffCommonUseFactory', '$timeout', parseDate]);
})();(function() {
    'use strict';

    var datePicker = function($log, contextualData) {
        var directive = {
            restrict: 'E',
            scope: {
                format: '@?',
                minValue: '=?',
                maxValue: '=?',
                datePickerSize: '=',
                minDatePlaceholder: '=',
                maxDatePlaceholder: '=',
                datePickerOptions: '=?',
                datePickerMode: '@?',
                minLabel: '@?',
                maxLabel: '@?',
                readOnly: '@?',
                isDisabled: '@?',
                enforceMinValue: '@?',
                enforceMaxValue: '@?',
                errorControl: '=?',
                parserSeparator: '@?'
            },
            controller: DatePickerController,
            controllerAs: 'dpc',
            bindToController: true,
            templateUrl: contextualData.context + '/modules/form-factory-core/javascript/angular/components/datePicker.html',
            link: linkFunc
        };

        return directive;

        function linkFunc(scope, el, attr, ctrl) {}
    };

    angular
        .module('formFactory')
        .directive('ffDatePicker', ['$log', 'contextualData', datePicker]);

    var DatePickerController = function($scope, $log, i18n, uibDateParser, $timeout, ffCommonUseFactory) {
        var dpc = this;
        var DEFAULT_FORMAT = 'YYYY-MM-DD';
        dpc.isOpenMin = false;
        dpc.isOpenMax = false;
        dpc.previousValue = {
            min: '',
            max: ''
        };
        dpc.selectedMinDate;
        dpc.selectedMaxDate;
        dpc.openDatePicker = openDatePicker;
        dpc.executeOnChange = executeOnChange;
        dpc.i18nMessageGetter = i18n.message;
        //Make sure format is set correctly.
        dpc.format = dpc.format == null || dpc.format.trim().length == 0 ? DEFAULT_FORMAT : dpc.format;
        //convert the format to be usable with uibDateParser, if it is set.
        dpc.uibDateParserFormat = convertFormatToUibDateFormat(dpc.format);
        activate();

        function activate() {
            //set which datepicker(s) we are displaying
            dpc.isAvailableDatepicker = {
                min: _.isUndefined(dpc.minValue) ? false : true,
                max: _.isUndefined(dpc.maxValue) ? false : true
            };
            //initialize errorMessage object
            if (_.isObject(dpc.errorControl)) {
                if (!_.isUndefined(dpc.errorControl.min) && _.isObject(dpc.errorControl.min)) {
                    dpc.errorControl.min.hasError = !_.isUndefined(dpc.errorControl.min.hasError) && _.isBoolean(dpc.errorControl.min.hasError) ? dpc.errorControl.min.hasError : false;
                } else {
                    dpc.errorControl.min = {
                        hasError: false,
                        errorMessage: null
                    }
                }
                if (!_.isUndefined(dpc.errorControl.max) && _.isObject(dpc.errorControl.max)) {
                    dpc.errorControl.max.hasError = !_.isUndefined(dpc.errorControl.max.hasError) && _.isBoolean(dpc.errorControl.max.hasError) ? dpc.errorControl.max.hasError : false;
                } else {
                    dpc.errorControl.max = {
                        hasError: false,
                        errorMessage: null
                    }
                }
            } else {
                dpc.errorControl = {
                    min : {
                        hasError: false,
                        errorMessage: null
                    },
                    max : {
                        hasError: false,
                        errorMessage: null
                    }
                }
            }
            dpc.minLabelAvailable = !_.isUndefined(dpc.minLabel);
            dpc.maxLabelAvailable = !_.isUndefined(dpc.maxLabel);
            dpc.readOnly = _.isUndefined(dpc.readOnly) ? false : (dpc.readOnly === true || dpc.readOnly == 'true');
            dpc.isDisabled = _.isUndefined(dpc.isDisabled) ? false : (dpc.isDisabled === true || dpc.isDisabled == 'true');
            dpc.enforceMinValue = _.isUndefined(dpc.enforceMinValue) ? false : (dpc.enforceMinValue === true || dpc.enforceMinValue == 'true');
            dpc.enforceMaxValue = _.isUndefined(dpc.enforceMaxValue) ? false : (dpc.enforceMaxValue === true || dpc.enforceMaxValue == 'true');
            //handle datepicker options
            setupDatepickerOptions();
            //initialize the corresponding date fields
            if (!_.isUndefined(dpc.minValue)) {
                if (dpc.minValue instanceof Date) {
                    dpc.selectedMinDate = dpc.minValue;
                } else if (moment(dpc.minValue, moment.ISO_8601, true).isValid()) {
                    dpc.selectedMinDate = new Date(moment(dpc.minValue, moment.ISO_8601, true).toISOString());
                } else {
                    var verifyDate = new Date(dpc.minValue);
                    if (dpc.minValue !== null && verifyDate instanceof Date && !isNaN(verifyDate.getTime())) {
                        dpc.selectedMinDate = verifyDate;
                    } else {
                        dpc.selectedMinDate = null;
                    }
                }
                dpc.initMin = !_.isUndefined(dpc.datePickerOptions.maxDate) && moment(dpc.datePickerOptions.maxDate, moment.ISO_8601, true).isBefore(new Date().toISOString()) ? moment(dpc.datePickerOptions.maxDate, moment.ISO_8601, true).toDate() : new Date();
            }
            if (!_.isUndefined(dpc.maxValue)) {
                if (dpc.maxValue instanceof Date) {
                    dpc.selectedMaxDate = new Date(dpc.maxValue.toISOString());
                } else if (moment(dpc.maxValue, moment.ISO_8601, true).isValid()) {
                    dpc.selectedMaxDate = new Date(moment(dpc.maxValue, moment.ISO_8601, true).toISOString());
                } else {
                    var verifyDate = new Date(dpc.maxValue);
                    if (dpc.maxValue !== null && verifyDate instanceof Date && !isNaN(verifyDate.getTime())) {
                        dpc.selectedMaxDate = verifyDate;
                    } else {
                        dpc.selectedMaxDate = null;
                    }
                }
                dpc.initMax = !_.isUndefined(dpc.datePickerOptions.maxDate) && moment(dpc.datePickerOptions.maxDate, moment.ISO_8601, true).isBefore(new Date().toISOString()) ? moment(dpc.datePickerOptions.maxDate, moment.ISO_8601, true).toDate() : new Date();
            }

            //Initialize the previous values for both min and max date, so that we have something to
            //revert to if the next selected date is not valid.
            dpc.previousValue.min = angular.copy(dpc.selectedMinDate);
            dpc.previousValue.max = angular.copy(dpc.selectedMaxDate);
        }

        function openDatePicker(popup) {
            dpc.isOpenMin = popup === 'min';
            dpc.isOpenMax = popup === 'max';
            if (dpc.isOpenMin) {
                $timeout(function() {
                    $timeout(function () {
                        //Focus the button toolbar first and then refocus the datepicker so that the full contents of the date picker is in view.
                        angular.element('div.datePickerMin').find('ul.uib-datepicker-popup button.uib-datepicker-current').focus();
                        angular.element('div.datePickerMin').find('ul.uib-datepicker-popup button.uib-datepicker-current').blur();
                        angular.element('div.datePickerMin').find('table').focus();
                    });
                });
            } else if(dpc.isOpenMax) {
                //Using 2 timeouts in order to execute the focus after the datepicker has already been focused.
                //This way our focus attempts will not be overridden.
                $timeout(function() {
                    $timeout(function () {
                        //Focus the button toolbar first and then refocus the datepicker so that the full contents of the date picker is in view.
                        angular.element('div.datePickerMax').find('ul.uib-datepicker-popup button.uib-datepicker-current').focus();
                        angular.element('div.datePickerMax').find('ul.uib-datepicker-popup button.uib-datepicker-current').blur();
                        angular.element('div.datePickerMax').find('table').focus();
                    });
                });
            }
        }

        /**
         * This function verifies if an options parameter has been passed
         * and removes any empty values from it
         * @function
         */
        function setupDatepickerOptions() {
            if (_.isUndefined(dpc.datePickerOptions)) {
                dpc.datePickerOptions = {};
                return;
            } else {
                //remove any uninitialized or empty settings to not cause any problems with ui datepicker
                _.each(dpc.datePickerOptions, function(option, optionName) {
                    if (_.isUndefined(option) || _.isEmpty(option)) {
                        delete dpc.datePickerOptions[optionName];
                    }
                });
            }

        }

        /**
         * This function will enforce min and max values if in options
         * and keep track of previously selected valid values
         * @function
         * @param input the input type that has changed (min or max)
         */
        function executeOnChange(input) {
            /** Make sure if this option is selected that we reset the date if:
             *  The entered value is greater than max (in the case of min value)
             *  Or the entered value is less than min (in the case of max value)
             */
            if (dpc.enforceMinValue && input === 'min' && (_.isUndefined(dpc.selectedMinDate) || !(moment(dpc.selectedMinDate, moment.ISO_8601, true).isValid()))) {
                dpc.selectedMinDate = moment("1970","YYYY").toDate();
            }
            if (dpc.enforceMaxValue && input === 'max' && (_.isUndefined(dpc.selectedMaxDate) || !(moment(dpc.selectedMaxDate, moment.ISO_8601, true).isValid()))) {
                dpc.selectedMaxDate = moment().toDate();
            }
            if (input === 'min' && ffCommonUseFactory.parseDate(dpc.selectedMinDate, dpc.format) && !_.isUndefined(dpc.selectedMaxDate) && ffCommonUseFactory.parseDate(dpc.selectedMaxDate, dpc.format)) {
                if (moment(dpc.selectedMinDate).isAfter(moment(dpc.selectedMaxDate).toISOString())) {
                    dpc.selectedMinDate = uibDateParser.parse(new Date(dpc.previousValue.min));
                }
            }
            if (input === 'max' && ffCommonUseFactory.parseDate(dpc.selectedMaxDate, dpc.format) && !_.isUndefined(dpc.selectedMinDate) && ffCommonUseFactory.parseDate(dpc.selectedMinDate, dpc.format)) {
                if (moment(dpc.selectedMaxDate).isBefore(moment(dpc.selectedMinDate).toISOString())) {
                    dpc.selectedMaxDate = uibDateParser.parse(new Date(dpc.previousValue.max));
                }
            }
            //Keep track of previous valid value. If user free types in date field and the value is not a valid date
            //it will result in undefined, so make sure this is not the case.
            if (input === 'min' && ffCommonUseFactory.parseDate(dpc.selectedMinDate, dpc.format)) {
                //set previous min value as long as date is valid.
                dpc.previousValue.min = angular.copy(dpc.selectedMinDate);
            }
            if (input === 'max' && ffCommonUseFactory.parseDate(dpc.selectedMaxDate, dpc.format)) {
                //set previous max value as long as date is valid.
                dpc.previousValue.max = angular.copy(dpc.selectedMaxDate);
            }
            //Set the binding to the value of the selected min date picker,
            //if the binding exists.
            if (input === 'min' && !_.isUndefined(dpc.selectedMinDate)) {
                $timeout(function() {
                    $scope.$apply(function() {
                        dpc.minValue = dpc.selectedMinDate;
                    })
                })
            }
            //Set the binding to the value of the selected max date picker,
            //if the binding exists.
            if (input === 'max' && !_.isUndefined(dpc.selectedMaxDate)) {
                $timeout(function() {
                    $scope.$apply(function() {
                        dpc.maxValue = dpc.selectedMaxDate;
                    })
                })
            }
        }

        function convertFormatToUibDateFormat(format) {
            if (format == null || format.trim().length == 0) {
                format = DEFAULT_FORMAT;
            }
            var separator = dpc.parserSeparator != null && dpc.parserSeparator.trim().length > 0 ? [dpc.parserSeparator] : ['-', '/'];
            var uibFormat = '';
            var separatorDetected = false;
            for (var i in separator) {
                //Go with the first separator we find, as this format should never have more than one type of separator.
                if (format.indexOf(separator[i]) !== -1) {
                    var sections = format.split(separator[i]);
                    for (var j in sections) {
                        if (sections[j].toLowerCase().indexOf('y') !== -1) {
                            uibFormat += sections[j].toLowerCase() + (j < sections.length-1 ? separator[i] : '');
                        } else if (sections[j].toLowerCase().indexOf('m') !== -1) {
                            uibFormat += sections[j].toUpperCase() + (j < sections.length-1 ? separator[i] : '');
                        } else if (sections[j].toLowerCase().indexOf('d') !== -1) {
                            uibFormat += sections[j].toLowerCase() + (j < sections.length-1 ? separator[i] : '');
                        }
                    }
                    separatorDetected = true;
                    break;
                }
            }
            if (!separatorDetected) {
                //Use value as is. (no separator is used)
                if (format.toLowerCase().indexOf('y') !== -1) {
                    uibFormat += format.toLowerCase();
                } else if (format.toLowerCase().indexOf('m') !== -1) {
                    uibFormat += format.toUpperCase();
                } else if (format.toLowerCase().indexOf('d') !== -1) {
                    uibFormat += format.toLowerCase();
                }
            }
            return uibFormat;
        }
    };

    DatePickerController.$inject = ['$scope', '$log', 'i18nService', 'uibDateParser', '$timeout', 'ffCommonUseFactory'];

    var dateValidator = function ($log, ffCommonUseFactory) {
        var directive = {
            restrict: 'A',
            require: ['^ngModel'],
            link: linkFunc
        };

        return directive;

        function linkFunc(scope, el, attr, ctrl) {
            var model = ctrl[0];
            var fieldId = attr.fieldId;
            var caseName = attr.caseName;
            model.$validators[attr.ffDateValidator] = function() {
                var isValid = ffCommonUseFactory.parseDate(model.$viewValue);
                scope.$emit('requiredValueChanged', {isValid: isValid, fieldId: fieldId, caseName: caseName, validator: attr.ffDateValidator});
                return isValid;
            };
        }
    };

    angular
        .module('formFactory')
        .directive('ffDateValidator', ['$log', 'ffCommonUseFactory', dateValidator]);
})();(function () {

	var editInPlace = function () {
		var directive = {
			restrict: 'E',
			scope: {
				value: '='
			},
			template: '<span ng-click="edit()" ng-bind="value"></span><input class="form-control" ng-model="value" />',
			link: linkFunction
		};

		return directive;

		function linkFunction(scope, el, attrs) {
			var inputElement = el.find('input');
			el.addClass('edit-in-place');
			el.addClass('inputHidden');

			scope.edit = function () {
				el.addClass('spanHidden');
				el.removeClass("inputHidden");
				inputElement[0].focus();
			};

			inputElement.on('blur', function (e) {
				e.stopPropagation();
				el.removeClass('spanHidden');
				el.addClass("inputHidden");
			});
		}
	};

	angular.module('formFactory')
		.directive('ffEditInPlace', [editInPlace]);
})();
(function () {
	'use strict';
	var logicsFieldValidation = function ($log, contextualData, $timeout) {
		var directive = {
			restrict: 'EA',
			require: ['^ngModel'],
			link: linkFunc
		};

		return directive;

		function linkFunc(scope, el, attr, ctrl) {
			var type = attr['ffFieldValueValidation'];
			var fieldId = attr['fieldId'];
			var caseName = attr['caseName']; //i.e. name of logic or validation

			el.bind('keyup', function (e) {
				scope.$apply(function () {
					//Validate current value
					validate(e.currentTarget.value);
				});
			});

			el.bind('change', function (e) {
				scope.$apply(function () {
					//Validate current value
					validate(e.currentTarget.value);
				});
			});

			//Validate when template loads, need to wait before bindings update
			$timeout(function () {
				validate(ctrl[0].$viewValue)
			}, 500);

			function validate(value) {
				switch (type) {
					case 'any' :
						if (value === undefined || value === "" || value === null) {
							scope.$emit('requiredValueChanged', {isValid: false, case: type, fieldId: fieldId, caseName: caseName});
						}
						else {
							scope.$emit('requiredValueChanged', {isValid: true, case: type, fieldId: fieldId, caseName: caseName});
						}
						break;
					case 'number' :
						if (value === undefined || value === "" || value === null || isNaN(value)) {
							scope.$emit('requiredValueChanged', {isValid: false, case: type, fieldId: fieldId, caseName: caseName});
						}
						else {
							scope.$emit('requiredValueChanged', {isValid: true, case: type, fieldId: fieldId, caseName: caseName});
						}
						break;
					case 'positiveNumber' :
						if (value === undefined || value === "" || value === null || isNaN(value) || value < 0) {
							scope.$emit('requiredValueChanged', {isValid: false, case: type, fieldId: fieldId, caseName: caseName});
						}
						else {
							scope.$emit('requiredValueChanged', {isValid: true, case: type, fieldId: fieldId, caseName: caseName});
						}
						break;
					case 'multiple' :
						scope.$emit('requiredValueChanged', {isValid: scope.vsc.selectedValues.length > 0, case: type, fieldId: fieldId, caseName: caseName});
						break;
				}
			}
		}
	};

	angular
		.module('formFactory')
		.directive('ffFieldValueValidation', ['$log', 'contextualData', '$timeout', logicsFieldValidation]);
})();
(function() {
    'use strict';
    /**
     * This service will resolve the template of the corresponding directive
     * @class
     * @private
     * @memberOf ff
     */
    var ResolveTemplatesService = function($log) {
        /**
         * Resolves the template path for the current directive
         * @param path the path for the provided template
         * @param viewType the defined view type that will be inserted into the path
         * @returns {*}
         */
        this.resolveTemplatePath = function(path, viewType) {
            //if view type is undefined it'll be set to an empty string in order to use the default view.
            viewType = viewType !== undefined ? '.' + viewType : '';
            return path.replace('.html.ajax', viewType + '.html.ajax');
        };
    };

    ResolveTemplatesService.$inject = ['$log'];

    angular
        .module('formFactory.templateResolver', [])
        .service('ffTemplateResolver', ResolveTemplatesService);
})();
(function() {
    'use strict';
    var sanitize = function($sce) {
        return function(htmlCode) {
            return $sce.trustAsHtml(htmlCode);
        };
    };

    var removeSpaces = function() {
        return function(string) {
            if (!angular.isString(string)) {
                return string;
            }
            return string.replace(/[\s]/g, '');
        };
    };

    var lowercase = function() {
        return function(string) {
            if (!angular.isString(string)) {
                return string;
            }
            return string.toLowerCase();
        };
    };

    var normalize = function() {
        return function(value) {
            var newValue = '';
            for (var i = 0; i < value.length; i++) {
                if (value.charAt(i) === '-') {
                    if (i++ < value.length) {
                        newValue += value.charAt(i).toUpperCase();
                    }
                    continue;
                }
                newValue += value.charAt(i);
            }
            return newValue;
        };
    };

    var removeDashes = function() {
        return function(string) {
            if (!angular.isString(string)) {
                return string;
            }
            return string.replace(/[\-]/g, '');
        };
    };

    var callUnderscoreString = function() {
        return function(str, fn, params) {
            str = str || '';
            params = params || [];
            params.unshift(str);
            return fn ? s[fn].apply(this, params) : str;
        };
    };

    var momentCalendar = function() {
        return function(string) {
            if (!angular.isString(string)) {
                return string;
            }
            return moment(string,'LLL').calendar();
        };
    };

    var resourceBundle = function(i18n) {
        return function(string, resourcePartial) {
            return i18n.message(resourcePartial + '.' + string);
        };
    };

    var removeChar = function() {
        return function(string, charIndex) {
            return string.substr(charIndex);
        };
    };

    var sanitizeFormName = function() {
        /**
         * Replaces characters that are not valid for angular form name attr
         * @returns sanitized form name
         */
        return function(formName) {
            if (!angular.isString(formName)) {
                return formName;
            }
            return formName.replace(/[`\s\'\"~+=!*&%#@,:;\.\^\$\?<\>\|\/\\\[\]\(\)\}\{\-]/g, '');
        };
    };

    var roundValue = function() {
        /**
         * Rounds the value to the nearest integer
         * @returns rounded up/down value
         */
        return function(value) {
            var regexp = new RegExp(/^-{0,1}\d*\.?\d*$/);

            if (!regexp.test(value)) {
                return value;
            }
            return Math.round(value);
        };
    };

    var dasherize = function() {
        return function(str) {
            return str != null ? str.toLowerCase().replace(/\s/g, '-') : '';
        };
    };

    angular.module('formFactory')
        .filter('sanitize', ['$sce', sanitize])
        .filter('removeSpaces', [removeSpaces])
        .filter('lowercase', [lowercase])
        .filter('normalize', [normalize])
        .filter('removeDashes', [removeDashes])
        .filter('s', [callUnderscoreString])
        .filter('momentCalendar', [momentCalendar])
        .filter('i18n', ['i18nService', resourceBundle])
        .filter('removeIndex', [removeChar])
        .filter('sanitizeFormName', [sanitizeFormName])
        .filter('roundValue', [roundValue])
        .filter('dasherize', [dasherize]);
})();
(function() {
    'use strict';

    var i18nMessageKeyDirectiveFunction = function(i18nService) {
        return {
            restrict: 'A',
            link: function($scope, $element, $attrs) {
                var i18n;
                if (!$attrs.messageParams) {
                    i18n = i18nService.message($attrs.messageKey);
                } else {
                    i18n = i18nService.format($attrs.messageKey, $attrs.messageParams);
                }

                if ($attrs.messageAttr) {
                    // store the i18n in the specified element attr
                    $element.attr($attrs.messageAttr, i18n);
                } else {
                    // set the i18n as element text
                    $element.text(i18n);
                }
            }
        };
    };

    var i18nTranslateFilterFunction = function(i18nService) {
        return function(input) {
            return i18nService.message(input);
        };
    };

    angular.module('i18n', [])
        .service('i18nService', function() {
            this.message = function(key, fallback) {
                if (ffi18n && ffi18n[key]) {
                    return ffi18n[key];
                } else if (angular.isDefined(fallback)) {
                    return fallback;
                } else {
                    return '???' + key + '???';
                }
            };

            this.format = function(key, params) {
                var replacer = function(params) {
                    return function(s, index) {
                        return params[index] ? (params[index] === '__void__' ? '' : params[index]) : '';
                    };
                };

                if (params) {
                    if (ffi18n && ffi18n[key]) {
                        return ffi18n[key].replace(/\{(\w+)\}/g, replacer(params.split('|')));
                    } else {
                        return '???' + key + '???';
                    }
                } else {
                    return this.message(key);
                }
            };
        })

        .directive('messageKey', ['i18nService', i18nMessageKeyDirectiveFunction])

        .filter('translate', ['i18nService', i18nTranslateFilterFunction]);
})();
(function() {
    'use strict';
    var UnderscoreFactory = function() {
        return window._;
    };

    angular.module('formFactory')
        .factory('_', UnderscoreFactory);
})();
