package org.jahia.modules.formfactory.dsl

import groovy.json.JsonBuilder
import org.jahia.api.Constants
import org.jahia.data.templates.JahiaTemplatesPackage
import org.jahia.services.content.*
import org.jahia.services.content.nodetypes.ExtendedNodeType
import org.jahia.services.templates.TemplatePackageRegistry
import org.jahia.services.usermanager.JahiaUserManagerService
import org.jahia.settings.SettingsBean
import org.slf4j.Logger
import org.slf4j.LoggerFactory

import javax.jcr.RepositoryException

class InputHandler implements DSLHandler {
    private static final Logger logger = LoggerFactory.getLogger(InputHandler.class);
    def JCRTemplate jcrTemplate;
    def TemplatePackageRegistry packageRegistry
    def SettingsBean settingsBean

    def input(@DelegatesTo(FormFactoryWizardParser) Closure cl) {
        parseDeclaration(cl, "input")
    }

    def select(@DelegatesTo(FormFactoryWizardParser) Closure cl) {
        parseDeclaration(cl, "select")
    }

    def radio(@DelegatesTo(FormFactoryWizardParser) Closure cl) {
        parseDeclaration(cl, "radio")
    }

    def checkbox(@DelegatesTo(FormFactoryWizardParser) Closure cl) {
        parseDeclaration(cl, "checkbox")
    }

    def parseDeclaration(Closure cl, String tab) {
        def inputType = new FormFactoryWizardParser()
        def code = cl.rehydrate(inputType, this, this)
        code.resolveStrategy = Closure.DELEGATE_FIRST
        code()
        logger.info("Declared input structure: " + inputType.toString());
        def result = inputType.contentMap
        createWizard result, cl.owner.currentPackage, cl.owner.currentNodeType, tab
    }

    def registerProperties(wizardEntry, value) {
        if (value instanceof String) {
            wizardEntry.setProperty("jsonValue", value)
        } else if (value instanceof Map) {
            assert value["values"] != null

            def values = value["values"]
            if (values instanceof List) {
                def jsonValue = []
                values.eachWithIndex { label, idx ->
                    jsonValue[idx] = label
                }
                def json = new JsonBuilder()
                json jsonValue
                wizardEntry.setProperty("jsonValue", json.toString())
            } else if (values instanceof Map) {
                def jsonValue = []
                values.eachWithIndex { k, l, idx ->
                    jsonValue[idx] = [key: k, value: l]
                }
                def json = new JsonBuilder()
                json jsonValue
                wizardEntry.setProperty("jsonValue", json.toString())
            }
        } else if (value instanceof List) {
            wizardEntry.setProperty("jsonValue", value[1])
        }
    }

    def createWizard(Map<String, Object> map, JahiaTemplatesPackage currentPackage, ExtendedNodeType currentNodeType, String inputTab) {
        assert map.containsKey("label")
        assert map.containsKey("template")
        assert map.containsKey("wizard")
        jcrTemplate.doExecuteWithSystemSessionAsUser(JahiaUserManagerService.instance.lookupRootUser().jahiaUser, Constants.EDIT_WORKSPACE, Locale.ENGLISH, new JCRCallback() {
            @Override
            Object doInJCR(JCRSessionWrapper jcrSessionWrapper) throws RepositoryException {
                JCRNodeWrapper bundleRootNode = jcrSessionWrapper.getNode(currentPackage.rootFolderPath + "/" + currentPackage.getVersion().toString())
                def rNodeWrapper = bundleRootNode.getNode("templates").getNode("contents")
                if (!rNodeWrapper.hasNode("form-factory-definitions")) {
                    rNodeWrapper.addNode("form-factory-definitions","jnt:contentFolder")
                }
                def inputsRootNode = rNodeWrapper.getNode("form-factory-definitions")
                def nodeName = JCRContentUtils.generateNodeName(map["label"], 32)

                if (inputsRootNode.hasNode(nodeName)) {
                    //Reintroduce check for development if necessary
                    /*if (settingsBean.isDevelopmentMode()) {
                        inputsRootNode.getNode(nodeName).remove()
                        jcrSessionWrapper.save()
                    } else {
                        return null;
                    }*/
                    return null;
                }

                def node = inputsRootNode.addNode(nodeName, currentNodeType.name)
                node.addMixin("fcmix:extendedDefinition")
                node.addMixin("fcmix:layoutUUID")
                node.setProperty(Constants.JCR_TITLE, map["label"])
                map.remove("label")
                node.setProperty("tab", inputTab)
                node.setProperty("template", map["template"])
                map.remove("template")
                node.setProperty("wizard", map["wizard"])
                map.remove("wizard")
                node.setProperty("layoutUUID", "fakeUUID")
                if (map.containsKey("supportedValidationTypes")) {
                    def supportedValidationTypes
                    if (map.get("supportedValidationTypes") instanceof List) {
                        supportedValidationTypes = (List<String>) map.get("supportedValidationTypes")
                    } else {
                        supportedValidationTypes = [map.get("supportedValidationTypes")]
                    }
                    node.setProperty("supportedValidationTypes", supportedValidationTypes.toArray(new String[supportedValidationTypes.size()]))
                    map.remove("supportedValidationTypes")
                }

                if (map.containsKey("supportedLogicTypes")) {
                    def supportedLogicTypes
                    if (map.get("supportedLogicTypes") instanceof List) {
                        supportedLogicTypes = (List<String>) map.get("supportedLogicTypes")
                    } else {
                        supportedLogicTypes = [map.get("supportedLogicTypes")]
                    }
                    node.setProperty("supportedLogicTypes", supportedLogicTypes.toArray(new String[supportedLogicTypes.size()]))
                    map.remove("supportedLogicTypes")
                }

                if (map.containsKey("choiceField")) {
                    node.setProperty("choiceField", map["choiceField"])
                    map.remove("choiceField")
                }

                if (map.containsKey("properties")) {
                    def properties = map["properties"]
                    properties.each { key, value ->
                        def wizardEntry = node.addNode(key, "fcnt:definitionOptions")
                        registerProperties(wizardEntry, value)
                    }
                    map.remove("properties")
                }

                if (map.containsKey("propertiesI18n")) {
                    def propertiesI18n = map["propertiesI18n"]
                    propertiesI18n.each { key, value ->
                        def wizardEntry = node.addNode(key, "fcnt:definitionOptionsTranslatable")
                        registerProperties(wizardEntry, value)
                    }
                    map.remove("propertiesI18n")
                }

                jcrSessionWrapper.save()
                return null
            }
        })
    }
}

